/*
 * g++: g++ -O2 -o ansearch ansearch.cpp
 * VC : cl /D_WIN32_ /EHsc /Ox /Feansearch.exe ansearch.cpp
*/

#include <cstdlib>
#include <cstring>
#include <memory>
#include <cstdio>
#include <fstream>
#include <iostream>
#include <cmath>
#include <string>
#include <sstream>
#include <stack>
#include <vector>
#include <set>
#include <map>
#include <algorithm>
using namespace std;


map<string, set<string> > dict;
int tnw;

#ifdef _WIN32_

#include <windows.h>
vector<string> enum_target_file() {
	WIN32_FIND_DATA wfd;
	vector<string> vs;
	
	HANDLE hFind = FindFirstFile("*", &wfd);
	if(hFind != INVALID_HANDLE_VALUE) {
		do {
			string s = wfd.cFileName;
			if(isdigit(s[s.size()-1]) && isdigit(s[s.size()-2]))
				vs.push_back(s);
		} while(FindNextFile(hFind, &wfd));
		FindClose(hFind);
	}
	return vs;
}

#else

#include <glob.h>
vector<string> enum_target_file() {
	glob_t g;
	int ret,i;
	vector<string> vs;
	
	ret = glob("*.[0-9][0-9]",0, NULL, &g);
	for(i=0;i<g.gl_pathc;i++) vs.push_back(g.gl_pathv[i]);
	globfree(&g);
	return vs;
}
#endif

void single_word(string word){
	sort(word.begin(),word.end());
	
	printf("------------------------\n");
	if(dict.find(word)==dict.end()) {
		printf("Single word : 0 words\n");
	}
	else {
		printf("Single word : %d words\n", dict[word].size());
		for(set<string>::iterator it=dict[word].begin(); it!=dict[word].end(); it++)
			cout << "  " << *it << endl;
	}
}

void double_word(string word){
	int L=word.size(),mask;
	sort(word.begin(),word.end());
	
	printf("------------------------\n");
	if(L==1) return (void)printf("sorry, too short.\n");
	if(L>20) return (void)printf("sorry, too long word.\n");
	
	
	set<string> did;
	for(mask=1;mask<1<<(L-1);mask++) {
		int i;
		string fm,la;
		for(i=0; i<L; i++) {
			if(mask & (1<<i)) fm += word[i];
			else la += word[i];
		}
		
		if(did.find(fm)!=did.end()) continue;
		did.insert(fm);
		did.insert(la);
		if(dict.find(fm)==dict.end() || dict.find(la)==dict.end()) continue;
		printf("  %s : %s : %d*%d entries\n", fm.c_str(),la.c_str(), dict[fm].size(), dict[la].size());
		for(set<string>::iterator it=dict[fm].begin(); it!=dict[fm].end(); it++)
			cout << "    fm:" << *it << endl;
		for(set<string>::iterator it=dict[la].begin(); it!=dict[la].end(); it++)
			cout << "    la:" << *it << endl;
	}
}

void triple_word(string word){
	int L=word.size(),mask;
	sort(word.begin(),word.end());
	int pow3[16],i;
	
	printf("------------------------\n");
	if(L<3) return (void)printf("sorry, too short.\n");
	if(L>15) return (void)printf("sorry, too long word.\n");
	
	pow3[0]=1;
	for(i=0;i<15;i++) pow3[i+1]=pow3[i]*3;
	
	set<string> did;
	for(mask=0;mask<pow3[L-1];mask++) {
		int i;
		string str[3];
		int numnum[3];
		numnum[0]=numnum[1]=numnum[2]=0;
		for(i=0; i<L; i++) numnum[mask/pow3[i]%3]++;
		if(numnum[0]==0 || numnum[1]==0 || numnum[2]==0) continue;
		
		for(i=0; i<L; i++) str[mask/pow3[i]%3] += word[i];
		sort(str,str+3);
		
		if(did.find(str[0]+" "+str[1])!=did.end()) continue;
		did.insert(str[0]+" "+str[1]);
		
		if(dict.find(str[0])==dict.end()) continue;
		if(dict.find(str[1])==dict.end()) continue;
		if(dict.find(str[2])==dict.end()) continue;
		printf("  %s : %s : %s : %d*%d*%d entries\n",
			str[0].c_str(),str[1].c_str(),str[2].c_str(),
			dict[str[0]].size(),dict[str[1]].size(),dict[str[2]].size());
		
		for(i=0;i<3;i++)
			for(set<string>::iterator it=dict[str[i]].begin(); it!=dict[str[i]].end(); it++)
				cout << "    " << i << ":" << *it << endl;
	}
}



void read_dict() {
	vector<string> vs = enum_target_file();
	int ret,i;
	
	for(i=0;i<vs.size();i++) {
		ifstream ifs(vs[i].c_str());
		string word,s;
		int nw=0;
		while(ifs && getline(ifs,word)) {
			int ng=0,j;
			word.erase(remove(word.begin(), word.end(), '\r'), word.end());
			word.erase(remove(word.begin(), word.end(), '\n'), word.end());
			for(j=0; j<word.size(); j++) if(word[j]<'a' || word[j]>'z') ng++;
			if(!ng) {
				s=word;
				sort(s.begin(),s.end());
				if(dict[s].find(word)==dict[s].end()) nw++, dict[s].insert(word);
			}
		}
		tnw += nw;
		printf("%s : %d words\n",vs[i].c_str(),nw);
	}
	printf("--total : %d words\n",tnw);
	
}


int main(int argc,char** argv){
	int i;
	if(argc<=1) {
		printf("usage: %s word\n",argv[0]);
		return 0;
	}
	
	read_dict();
	for(i=1;i<argc;i++) {
		printf("word : %s\n",argv[i]);
		single_word(string(argv[i]));
		double_word(string(argv[i]));
		triple_word(string(argv[i]));
	}
	
	return 0;
}


